@@ -1,7 +1,6 @@
 import sklearn.model_selection
 import time
-from directencoder import DirectEncoder
-from compressingencoder import CompressingEncoder
+from lcdb.directencoder import DirectEncoder
 
 import numpy as np
 import pandas as pd
@@ -17,6 +16,10 @@ from tqdm import tqdm
 
 import traceback
 
+import shutil, tarfile
+
+import importlib.resources as pkg_resources
+from io import StringIO
 
 def get_dataset(openmlid):
     ds = openml.datasets.get_dataset(openmlid)
@@ -214,40 +217,51 @@ def get_entry(learner_name, learner_params, X, y, anchor, outer_seed, inner_seed
     
     return info
 
-def get_curve_by_metric(json_curve_descriptor, metric, encoder = DirectEncoder(), error="raise"):
+def get_curve_for_metric_in_dataframe(json_curve_descriptor, metric, encoder = DirectEncoder(), error="raise", precision=4):
     
-    # gather data
-    anchors_tmp = []
-    values_train_tmp = []
-    values_valid_tmp = []
-    values_test_tmp = []
+    cols = ['size_train', 'size_test', 'outer_seed', 'inner_seed', 'traintime']
+    rows = []
     for entry in json_curve_descriptor:
         try:
             m_train, m_valid, m_test = get_metric_of_entry(entry, metric, encoder)
-            anchors_tmp.append(entry["size_train"])
-            values_train_tmp.append(m_train)
-            values_valid_tmp.append(m_valid)
-            values_test_tmp.append(m_test)
+            row = [entry["size_train"], entry["size_test"], entry["outer_seed"], entry["inner_seed"], entry["traintime"], np.round(m_train, precision), np.round(m_valid, precision), np.round(m_test, precision)]
+            rows.append(row)
         except:
             if error == "message":
                 print("Ignoring entry with exception!")
             else:
                 raise
+    return pd.DataFrame(rows, columns=cols + ["score_" + i for i in ["train", "valid", "test"]])
 
+
+def get_curve_from_dataframe(curve_df):
     
-    # convert all three lists in numpy arrays
-    anchors_tmp = np.array(anchors_tmp)
-    values_train_tmp = np.array(values_train_tmp)
-    values_valid_tmp = np.array(values_valid_tmp)
-    values_test_tmp = np.array(values_test_tmp)
+    # sanity check to see that, if we know the learner, that there is only one of them
+    if "learner" in curve_df.columns and len(pd.unique(curve_df["learner"])) > 1:
+        raise Exception("pass only dataframes with entries for a single learner.")
+    if "openmlid" in curve_df.columns and len(pd.unique(curve_df["openmlid"])) > 1:
+        raise Exception("pass only dataframes with entries for a single openmlid.")
     
-    # prepare data
-    anchors = sorted(list(np.unique(anchors_tmp)))
-    values_train = [list(values_train_tmp[anchors_tmp == v]) for v in anchors]
-    values_valid = [list(values_valid_tmp[anchors_tmp == v]) for v in anchors]
-    values_test = [list(values_test_tmp[anchors_tmp == v]) for v in anchors]
+    # gather data
+    anchors = sorted(list(pd.unique(curve_df["size_train"])))
+    values_train = []
+    values_valid = []
+    values_test = []
+    
+    # extract curve
+    for anchor in anchors:
+        curve_df_anchor = curve_df[curve_df["size_train"] == anchor]
+        values_train.append(list(curve_df_anchor["score_train"]))
+        values_valid.append(list(curve_df_anchor["score_valid"]))
+        values_test.append(list(curve_df_anchor["score_test"]))
+        
     return anchors, values_train, values_valid, values_test
+
+def get_curve_by_metric_from_json(json_curve_descriptor, metric, encoder = DirectEncoder(), error="raise"):
     
+    return get_curve_from_dataframe(get_curve_for_metric_in_dataframe(json_curve_descriptor, metric, encoder, error))
+
+        
         
 def get_metric_of_entry(entry, metric, encoder = DirectEncoder()):
     y_train = encoder.decode_label_vector(entry["y_train"])
@@ -274,7 +288,10 @@ def get_metric_of_entry(entry, metric, encoder = DirectEncoder()):
                 y_prob_train = np.zeros((len(y_hat_train), len(labels)))
                 for i, label in enumerate(y_hat_train):
                     y_prob_train[i,labels.index(label)] = 1
-                    y_prob_test = np.zeros((len(y_hat_test), len(labels)))
+                y_prob_valid = np.zeros((len(y_hat_valid), len(labels)))
+                for i, label in enumerate(y_hat_valid):
+                    y_prob_valid[i,labels.index(label)] = 1
+                y_prob_test = np.zeros((len(y_hat_test), len(labels)))
                 for i, label in enumerate(y_hat_test):
                     y_prob_test[i,labels.index(label)] = 1
             m = sklearn.metrics.log_loss
@@ -356,4 +373,61 @@ def plot_train_and_test_curve(curve, ax = None):
     ax.fill_between(anchors, np.mean(curve[2][-1]) - 0.0025, np.mean(curve[2][-1]) + 0.0025, color="black", alpha=0.1, hatch=r"//")
     
     if fig is not None:
-        plt.show()
\ No newline at end of file
+        plt.show()
+
+        
+'''
+    this recovers the database table from a tar ball from the disk.
+    
+    The tar ball is unpacked, read, and the unpacked content is removed again
+'''
+def get_raw_df(openmlid):
+    
+    # extract db file
+    fname = f"database-original/lcdb-{openmlid}.tar.gz"
+    path = "./tmp/conversions"
+    tar = tarfile.open(fname, "r:gz")
+    tar.extractall(path=path)
+    tar.close()
+    
+    # read in data frame
+    df = pd.read_csv(f"{path}/lcdb-{openmlid}.csv", delimiter=";")
+    
+    # delete folder
+    shutil.rmtree(path)
+    
+    # return dataframe
+    return df
+
+'''
+    this takes a dataframe modeling the MySQL table and turns it into the final format (result column is resolved)
+'''
+def convert_raw_df_into_final_df(df_raw, metric):
+    cols = ["openmlid", "learner", "size_train", "size_test", "outer_seed", "inner_seed", "traintime", "score_train", "score_valid", "score_test"]
+    df_results = pd.DataFrame([], columns=cols)
+    for i, row in df_raw.iterrows():
+        curve_raw = json.loads(row["result"])
+        curve_df = get_curve_for_metric_in_dataframe(curve_raw, metric)
+        curve_df["learner"] = row["learner"]
+        curve_df["openmlid"] = row["openmlid"]
+        df_results = pd.concat([df_results, curve_df[cols]])
+    return df_results
+
+'''
+    this takes a series of dataset descriptors and computes a full dataframe with all learning curves from the tar balls
+    
+    for every openmlid, there must be the file "database-original/lcdb-<id>.tar.gz" available
+'''
+def compile_dataframe_for_all_datasets_on_metric(openmlids, metric):
+    df_results = None
+    for openmlid in tqdm(openmlids):
+        df_results_tmp = convert_raw_df_into_final_df(get_raw_df(openmlid), metric)
+        df_results = df_results_tmp if df_results is None else pd.concat([df_results, df_results_tmp])
+    return df_results
+
+
+def get_all_curves_for_metric(metric="accuracy"):
+    supported_metrics = ["accuracy", "logloss"]:
+    if not metric in supported_metrics:
+        raise ValueError(f"Unsupported metric {metric}. Supported metrics are {supported_metrics}")
+    return pd.read_csv(StringIO(pkg_resources.read_text('lcdb', f'database-{metric}.csv')))
\ No newline at end of file
